/*****************************************************************************
 **   FILE NAME    : ifx_lldp_config.h
 **   PROJECT      : LLDP
 **   MODULES      : Config Module
 **   SRC VERSION  : V1.0
 **
 **   DATE         : 14-04-2007
 **   AUTHOR       : Awanish G, Supreet M S, Pavan G M.
 **   DESCRIPTION  : This file contains all configurable parameters.
 **   FUNCTIONS    :
 **   COMPILER     :
 **   REFERENCE    : Coding guide lines for VSS, DIS of CCS.
 **   COPYRIGHT    : Infineon Technologies AG 2003 - 2004
 **
 **  Version Control Section
 **   $Author$
 **   $Date$
 **   $Revisions$
 **   $Log$       Revision history
 ****************************************************************************/
#ifndef	__IFX_LLDP_CONFIG_H__
#define	__IFX_LLDP_CONFIG_H__

/* LLDP Manager FIFO */
#define IFX_LLDP_MS_FIFO "/tmp/LldpManagerFifo"

#ifndef DEBUG_ENABLED
#define printf(...)
#endif

#define	PRINT printf

#define TRUE 1
#define FALSE 0

/* Possible return values from interface APIs */
#define IFX_INTF_SUCCESS 0
#define IFX_INTF_FAIL -1

/* Maximum remote MIB entries allowed */
#define IFX_LLDP_MAX_REMOTE_MIB_PER_PORT 50

/* Maximum number of port supported in this release */
#define IFX_LLDP_MAX_PORTS 2

/* Maximum frame size of an LLDP packet */
#define IFX_LLDP_MAX_FRAME_SIZE 1024

/* Maximum number of Network Policy TLV's supported */
#define IFX_LLDP_MAXNETWORKTLVS 8

/* MAC/PHY MAU Type for PC */
#define IFX_LLDP_MAU_TYPE_PC 4

/* MAC/PHY MAU Type for LAN */
#define IFX_LLDP_MAU_TYPE_LAN 2

#define IANA_IPV4_NETWORK_ADDRESS_TYPE 1
#define IANA_IPV6_NETWORK_ADDRESS_TYPE 2
 
/* This enum defines all possible values of Chassis Id subtype. Refer to 802.1 AB, 
   section 9.5.2 for details */
enum
{
  CHASSISID_SUBTYPE_RESERVED = 0,
  CHASSISID_SUBTYPE_CHASSIS_COMPONENT,
  CHASSISID_SUBTYPE_INTERFACE_ALIAS,
  CHASSISID_SUBTYPE_PORT_COMPONENT,
  CHASSISID_SUBTYPE_MAC_ADDRESS,
  CHASSISID_SUBTYPE_NETWORK_ADDRESS,
  CHASSISID_SUBTYPE_INTERFACE_NAME,
  CHASSISID_SUBTYPE_LOCALLY_ASSIGNED,
  CHASSISID_SUBTYPE_EXT_RESERVED
};

/* This enum defines all possible values of Port Id subtype. Refer to 802.1 AB, 
   section 9.5.3 for details */
enum
{
  PORTID_SUBTYPE_RESERVED = 0,
  PORTID_SUBTYPE_INTERFACE_ALIAS,
  PORTID_SUBTYPE_PORT_COMPONENT,
  PORTID_SUBTYPE_MAC_ADDRESS,
  PORTID_SUBTYPE_NETWORK_ADDRESS,
  PORTID_SUBTYPE_INTERFACE_NAME,
  PORTID_SUBTYPE_AGENT_CIRCUIT_ID,
  PORTID_SUBTYPE_LOCALLY_ASSIGNED,
  PORTID_SUBTYPE_EXT_RESERVED
};

/* Hard coded Chassis Id subtype for this release. Refer to 802.1 AB, section
   9.5.2 for details */
#define IFX_LLDP_CHASSIS_ID_SUBTYPE CHASSISID_SUBTYPE_NETWORK_ADDRESS 

/* Hard coded Port Id subtype for this release. Refer to 802.1 AB, section
   9.5.3 for details */
#define IFX_LLDP_PORT_ID_SUBTYPE PORTID_SUBTYPE_MAC_ADDRESS

#endif	/* __IFX_LLDP_CONFIG_H__ */
