
#ifndef LLDP_TX_MODULE_H
#define LLDP_TX_MODULE_H

/* Forward declarations */
struct lldp_event;
struct lldp_port;
struct lldp_tlv_list;

/*This structure maintains the LLDP Tx module operational statistics 
  counters */
struct lldp_tx_port_statistics {
  uint32 statsFramesOutTotal;
};

/*This structure maintains the LLDP Transmit state machine timers*/
struct lldp_tx_port_timers {
    /* Transmit when this expires */
    uint32 txTTR;
    uint32 txShutdownWhile;
    uint32 txDelayWhile;
};

/* This structure represents local data of the entire LLDP Transmit module and
   Transmit State Machine*/
typedef struct {
  uint8 *frame;
  uint32 sendsize;
  boolean initpacket;
  lldp_tx_states state;
  int8 currstate; //has to be init to -1
  uint8 somethingChangedLocal;
  uint16 txTTL;
  uint32 medFastStart; /*indicates Fast Start mechanism is active*/
  struct lldp_tx_port_timers timers;
  struct lldp_tx_port_statistics statistics;
  uint16 unInterval;/*timer id1*/
  uint16 unDelay; /*timer id2*/
  uint16 unShutDown;/*timer id3*/
}lldp_tx_port;

/* Tx module interface functions */
void LLdp_tx_HandleEvent(uint16 uiPortIndex,x_IFX_LLDP_Lldp_Event *event);
void LLdp_tx_UpdateMIBStats(struct lldp_port *lldp_port);
int32 LLdp_tx_cleanup(struct lldp_port *lldp_port);

/****Functions Defined by the 802.1AB specification****/
uint8 txInitializeLLDP(struct lldp_port *lldp_port);
void mibConstrInfoLLDPDU(struct lldp_port *lldp_port);
void mibConstrShutdownLLDPDU();
uint8 txFrame();
void ConstrLLDPDUinfo(struct lldp_port *lldp_port,
                      struct lldp_tlv_list **tlvlist,
                      uint32 fram_off);
void ConstrLLDPheader(struct lldp_port *lldp_port,uint32 *frameoff);

#endif /* LLDP_TX_MODULE_H */
