/*******************************************************************
 *
 * OpenLLDP RX Statemachine
 *
 * Licensed under a dual GPL/Proprietary license.  
 * See LICENSE file for more info.
 * 
 * File: lldp_rx_sm.c
 * 
 * Authors: Terry Simons (terry.simons@gmail.com)
 *
 *******************************************************************/

/**************************************************************************
**   FILE NAME		: ifx_lldp_rx_sm.c
**   PROJECT		: Inca-Ip
**   MODULES	    : LLDP
**   SRC VERSION	: V1.0
**
**   DATE			:12-04-2007
**   AUTHOR			:Pavan GM
**   DESCRIPTION	:Source file 	
**   COMPILER		:GCC
**   REFERENCE		:Coding Guideline Version 2.0
**							
**   COPYRIGHT      :Copyright © 2004
**                   Infineon Technologies AG
**                   St. Martin Strasse 53; 81669 München, Germany
**   DESCLAIMER     :Any use of this Software is subject to the conclusion 
**                   of a respective License Agreement.
**                   Without such a License Agreement no rights to the 
**         			 Software are granted. 
**
**  Version Control Section 
**   $Author$
**   $Date$
**   $Revisions$
**   $Log$       Revision history
***********************************************************************/

#include <arpa/inet.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "ifx_common_defs.h"
#include "ifx_lldp_config.h"
#include "ifx_lldp_mib_if.h"
#include "ifx_lldp_cm.h"
#include "ifx_lldp_rx_sm.h"
#include "ifx_lldp_tx_sm.h"
#include "ifx_lldp_rx_module.h"
#include "ifx_lldp_tx_module.h"
#include "ifx_lldp_port.h"
#include "ifx_lldp_tlv.h"
#include "ifx_lldp_timer_if.h"
#include "ifx_lldp_intf_defs.h"
#include "ifx_lldp_ms_if.h"

EXTERN struct lldp_port *vpxlldp_port;
/*this stores all the new recv msap and associated TimerID */
extern struct lldp_rx_msaptimermap *MsapTimer[IFX_LLDP_MAX_REMOTE_MIB_PER_PORT]; //in rx_module.c

/* This is an 802.1AB per-port variable, so it should go in the port structure */
uint8 badFrame;

/******************************************************************
*  Function Name    : rxChangeToState
*  Description      : This function updates the current RX FSM state in 
*		      lldp port st
*  Input Values     : lldp_port,current state
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void rxChangeToState(struct lldp_port *lldp_port, uint8 state)
{
  lldp_port->rx.state = state;
}

#if 0

void rxBadFrameInfo(uint8_t frameErrors)
{
//  debug_printf(DEBUG_NORMAL, "[WARNING] This frame had %d errors!\n", frameErrors);
}

char *rxStateFromID(uint8_t state)
{
  switch(state)
    {
    case LLDP_WAIT_PORT_OPERATIONAL:
      return "LLDP_WAIT_PORT_OPERATIONAL";
    case DELETE_AGED_INFO:
      return "DELETE_AGED_INFO";
    case RX_LLDP_INITIALIZE:
      return "RX_LLDP_INITIALIZE";
    case RX_WAIT_FOR_FRAME:
      return "RX_WAIT_FOR_FRAME";
    case RX_FRAME:
      return "RX_FRAME";
    case DELETE_INFO:
      return "DELETE_INFO";
    case UPDATE_INFO:
      return "UPDATE_INFO";
    };

//  debug_printf(DEBUG_NORMAL, "[ERROR] Unknown RX State: '%d'\n", state);
  return "Unknown";
}
#endif

/******************************************************************
*  Function Name    : rxGlobalStatemachineRun
*  Description      : This function updates the curr state on satisfying
*		                  prerequisite conditions.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void rxGlobalStatemachineRun(struct lldp_port *lldp_port)
{
  /* NB: IEEE 802.1AB Section 10.5.5.3 claims that */
  /* An unconditional transfer should occur when */
  /* "(rxInfoAge = FALSE) && (portEnabled == FALSE)" */
  /* I believe that "(rxInfoAge = FALSE)" is a typo and should be: */
  /* "(rxInfoAge == FALSE)" */
   PRINT("\n IN -> rxGlobalStatemachineRun\n ");
   if(lldp_port->rx.state == LLDP_WAIT_PORT_OPERATIONAL)
   {
     lldp_port->rx.currstate = lldp_port->rx.state ;
     if((lldp_port->rx.rxInfoAge == FALSE) && (lldp_port->portEnabled == FALSE))
     {
       rxChangeToState(lldp_port, LLDP_WAIT_PORT_OPERATIONAL);
       return;
     }
   }
  
  switch(lldp_port->rx.state)
    {
      case LLDP_WAIT_PORT_OPERATIONAL:
      {
        PRINT("\n IN case LLDP_WAIT_PORT_OPERATIONAL\n ");
        lldp_port->rx.currstate = lldp_port->rx.state ;
        /*when up if any remote Msap expires first delete that.This is imp.should
	 *not keep expired values with us XXX ????*/
	      if(lldp_port->rx.rxInfoAge == TRUE)
	  			rxChangeToState(lldp_port, DELETE_AGED_INFO);
				else if(lldp_port->portEnabled == TRUE)
	  			rxChangeToState(lldp_port, RX_LLDP_INITIALIZE);
        PRINT("\n out case LLDP_WAIT_PORT_OPERATIONAL\n ");
      }break;
      case DELETE_AGED_INFO:
      {
     		lldp_port->rx.currstate = lldp_port->rx.state ;
	      rxChangeToState(lldp_port, LLDP_WAIT_PORT_OPERATIONAL);
      }break;
      case RX_LLDP_INITIALIZE:
      {
     		lldp_port->rx.currstate = lldp_port->rx.state ;
        if(lldp_port->portEnabled == FALSE)
        {
   				rxChangeToState(lldp_port, LLDP_WAIT_PORT_OPERATIONAL);      
          break;
        }
				if((lldp_port->adminStatus == enabledRxTx) || (lldp_port->adminStatus == enabledRxOnly))
	  			rxChangeToState(lldp_port, RX_WAIT_FOR_FRAME);
      }break;
    	case RX_WAIT_FOR_FRAME:
      {
     		lldp_port->rx.currstate = lldp_port->rx.state ;
        if(lldp_port->portEnabled == FALSE)
        {
   				rxChangeToState(lldp_port, LLDP_WAIT_PORT_OPERATIONAL);      
          break;
        }

				if((lldp_port->adminStatus == enabledTxOnly) || (lldp_port->adminStatus == disabled))
	  			rxChangeToState(lldp_port, RX_LLDP_INITIALIZE);
				else if(lldp_port->rx.rxInfoAge == TRUE)
	  			rxChangeToState(lldp_port, DELETE_INFO);
				else if(lldp_port->rx.rcvFrame == TRUE)
	  			rxChangeToState(lldp_port, RX_FRAME);
      }break;
    	case DELETE_INFO:
    	{
     		lldp_port->rx.currstate = lldp_port->rx.state ;
        if(lldp_port->portEnabled == FALSE)
        {
   				rxChangeToState(lldp_port, LLDP_WAIT_PORT_OPERATIONAL);      
          break;
        }

				rxChangeToState(lldp_port, RX_WAIT_FOR_FRAME);
      }break;
    	case RX_FRAME:
      {
     		lldp_port->rx.currstate = lldp_port->rx.state ;
        if(lldp_port->portEnabled == FALSE)
        {
   				rxChangeToState(lldp_port, LLDP_WAIT_PORT_OPERATIONAL);      
          break;
        }
		    /*Check this cond first and break because if the received frame has TTL 0
	       *then just delete it.no need of going ahead XXX*/

				if(lldp_port->rx.timers.rxTTL == 0)
				{
	  			rxChangeToState(lldp_port, DELETE_INFO);
				}
				/*if changes update*/
				else if((lldp_port->rx.timers.rxTTL != 0) && (lldp_port->rx.rxChanges == TRUE))
				{
	    			rxChangeToState(lldp_port, UPDATE_INFO);
				}
				/*if bad frame or no changes in already existing MSAP then ignore.Check badframe var XXX ??*/
				else if((lldp_port->rx.badFrame == TRUE) || ((lldp_port->rx.timers.rxTTL!=0) && (lldp_port->rx.rxChanges == FALSE)))
				{
	    		rxChangeToState(lldp_port, RX_WAIT_FOR_FRAME);
				}
      }break;
    	case UPDATE_INFO:
      {
     		lldp_port->rx.currstate = lldp_port->rx.state ;
				if(lldp_port->portEnabled == FALSE)
        {
   				rxChangeToState(lldp_port, LLDP_WAIT_PORT_OPERATIONAL);      
          break;
        }

				rxChangeToState(lldp_port, RX_WAIT_FOR_FRAME);
      }break;
    default:
      //debug_printf(DEBUG_NORMAL, "[ERROR] The RX Global State Machine is broken!\n");
      break;
    }

   PRINT("\n OUT -> rxGlobalStatemachineRun\n ");
  return;
}

/******************************************************************
*  Function Name    : rxStatemachineRun
*  Description      : This function takes necessary actions for the 
*		      						particular RX FSM state.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void rxStatemachineRun(struct lldp_port *lldp_port)
{
   PRINT("\n IN -> rxStatemachineRun\n ");
  do {
  switch(lldp_port->rx.state)
    {
      case LLDP_WAIT_PORT_OPERATIONAL:
      {
	      rx_do_lldp_wait_port_operational(lldp_port);
        rxGlobalStatemachineRun(lldp_port);
      }break;
      case DELETE_AGED_INFO:
      {
        if(lldp_port->rx.currstate != lldp_port->rx.state)
	      		rx_do_delete_aged_info(lldp_port);
  			rxGlobalStatemachineRun(lldp_port);
      }break;
      case RX_LLDP_INITIALIZE:
      {
        if(lldp_port->rx.currstate != lldp_port->rx.state)
	      		rx_do_rx_lldp_initialize(lldp_port);
  			rxGlobalStatemachineRun(lldp_port);
      }break;
      case RX_WAIT_FOR_FRAME:
      {
        if(lldp_port->rx.currstate != lldp_port->rx.state)
	      		rx_do_rx_wait_for_frame(lldp_port);
  			rxGlobalStatemachineRun(lldp_port);
      }break;
    	case RX_FRAME:
      {
        if(lldp_port->rx.currstate != lldp_port->rx.state)
						rx_do_rx_frame(lldp_port);
  			rxGlobalStatemachineRun(lldp_port);
      }break;
    	case DELETE_INFO:
      {
        if(lldp_port->rx.currstate != lldp_port->rx.state)
						rx_do_rx_delete_info(lldp_port);
  			rxGlobalStatemachineRun(lldp_port);
      }break;
    	case UPDATE_INFO:
      {
        if(lldp_port->rx.currstate != lldp_port->rx.state)
					rx_do_rx_update_info(lldp_port);
  			rxGlobalStatemachineRun(lldp_port);
      }break;
    default:
      //debug_printf(DEBUG_NORMAL, "[ERROR] The RX State Machine is broken!\n");      
      break;
    }

  }while(lldp_port->rx.currstate != lldp_port->rx.state);

   PRINT("\n OUT -> rxStatemachineRun\n ");
}

/******************************************************************
*  Function Name    : rx_do_lldp_wait_port_operational
*  Description      : This function takes actions when RX FSM is in
*		      LLDP_WAIT_PORT_OPERATIONAL state.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void rx_do_lldp_wait_port_operational(struct lldp_port *lldp_port)
{
  if(lldp_port->rx.rxInfoAge != TRUE) 
			lldp_port->rx.rxInfoAge = FALSE;
}

/******************************************************************
*  Function Name    : rx_do_delete_aged_info
*  Description      : This function takes actions when RX FSM is in 
*                     DELETE_AGED_INFO state.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void rx_do_delete_aged_info(struct lldp_port *lldp_port)
{
  //LLDP_FUNCTION_DEBUG;

  int16 temp;
  x_IFX_LLDP_MAC RemMacAddr;
  struct LldpMib TempRemMIB;
  e_IFX_LLDP_DEVICE_TYPE RemDeviceType;

  /* As per IEEE 802.1AB 10.5.5.3 state diagram */
  lldp_port->rx.somethingChangedRemote = FALSE;

  /*Delete the entry for which TTL expired*/
  temp=lldp_port->rx.uiTempIndex;

  memcpy(&RemMacAddr.uiaMacAddress[0],&MsapTimer[temp]->Mac[0],6);
	getMibData(remoteMibMsap,lldp_port->uiPortIndex ,MsapTimer[temp]->Msap,(void *)&TempRemMIB);
		  
	if(TempRemMIB.med_cap.deviceType == 4)
        RemDeviceType=IFX_LLDP_NCD;
      else if(TempRemMIB.med_cap.deviceType == 0)
        RemDeviceType=-1;
   		else 
        RemDeviceType=IFX_LLDP_EPD;
      
  if(deleteMibData(lldp_port->uiPortIndex,MsapTimer[temp]->Msap)==MIB_SUCCESS)
  {
		/*Intimate LLDP Manager about expiry only if deleted*/
    IFX_LLDP_PACKET_EXPIRED(RemDeviceType,&RemMacAddr); 
  }
  else 
  	PRINT("\nMIB delete fail\n");

  /*delete all data in my array*/
  free(MsapTimer[temp]->Msap); //free msap
  MsapTimer[temp]->Msap = NULL;
  
  free(MsapTimer[temp]->Mac); //free mac
  MsapTimer[temp]->Mac = NULL;
     
  free(MsapTimer[temp]); //free entry
  MsapTimer[temp] = NULL;

 // mibDeleteObjects(lldp_port);
  lldp_port->rx.rxInfoAge = FALSE;
  lldp_port->rx.somethingChangedRemote = TRUE;
}

/******************************************************************
*  Function Name    : rx_do_rx_lldp_initialize 
*  Description      : This function takes actions when RX FSM is in 
*                     RX_LLDP_INITIALIZE state.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void rx_do_rx_lldp_initialize(struct lldp_port *lldp_port)
{
  /* As per IEEE 802.1AB 10.5.5.3 state diagram */
  rxInitializeLLDP(lldp_port);
  lldp_port->rx.rcvFrame = FALSE;

  //lldp_port->adminStatus = enabledRxTx;
}

/******************************************************************
*  Function Name    : rx_do_rx_wait_for_frame
*  Description      : This function takes actions when RX FSM is in 
*                     RX_WAIT_FOR_FRAME state.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void rx_do_rx_wait_for_frame(struct lldp_port *lldp_port)
{
  PRINT("\nIN->rx_do_rx_wait_for_frame\n");
  /* As per IEEE 802.1AB 10.5.5.3 state diagram */
  lldp_port->rx.badFrame = FALSE;
  lldp_port->rx.rxInfoAge = FALSE;
  lldp_port->rx.somethingChangedRemote = FALSE;
  
  /** cleanup the previous packet data***/

  //Reset all local var and rx stats here and get ready for next cycle.
  memset(&lldp_port->rx.LLDPRemoteMIB,0,sizeof(struct LldpMib));
  
  //nullify the currRecvmsap in rx
  if(lldp_port->rx.curr_recvmsap!= NULL)
  {
    free(lldp_port->rx.curr_recvmsap);
    lldp_port->rx.curr_recvmsap = NULL;
  }

  //nullify the currRecvmac in rx
  if(lldp_port->rx.curr_recvmac!= NULL)
  {
    free(lldp_port->rx.curr_recvmac);
    lldp_port->rx.curr_recvmac = NULL;
  }

  lldp_port->rx.uiTempIndex = -1;
  lldp_port->rx.statistics.statsAgeoutsTotal = 0; 
  lldp_port->rx.statistics.statsFramesDiscardedTotal = 0;
  lldp_port->rx.statistics.statsFramesInErrorsTotal = 0;
  lldp_port->rx.statistics.statsFramesInTotal = 0;
  lldp_port->rx.statistics.statsTLVsDiscardedTotal = 0;
  lldp_port->rx.statistics.statsTLVsUnrecognizedTotal = 0;
  PRINT("\nOUT->rx_do_rx_wait_for_frame\n");
}

/******************************************************************
*  Function Name    : rx_do_rx_frame
*  Description      : This function takes actions when RX FSM is in 
*                     RX_FRAME state.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void rx_do_rx_frame(struct lldp_port *lldp_port)
{
  PRINT("\nIN->rx_do_rx_frame\n");
  /* As per IEEE 802.1AB 10.5.5.3 state diagram */
  lldp_port->rx.rxChanges = FALSE;
  lldp_port->rx.rcvFrame = FALSE;
  lldp_port->rx.uiPresentMsap = FALSE;
  /*reset partial mib st built XXX ???*/
  //memset(&lldp_port->rx.LLDPRemoteMIB,0,sizeof(struct LldpMib)); 
  rxProcessFrame(lldp_port);
   
  // Clear the frame buffer out to avoid weird problems. ;)
  bzero(&lldp_port->rx.frame[0], lldp_port->mtu);
  PRINT("\nOUT->rx_do_rx_frame\n");
}

/******************************************************************
*  Function Name    : rx_do_rx_delete_info
*  Description      : This function takes actions when RX FSM is in 
*                     DELETE_INFO state.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void rx_do_rx_delete_info(struct lldp_port *lldp_port)
{
  //LLDP_FUNCTION_DEBUG;
  x_IFX_LLDP_MAC RemMacAddr;
  struct LldpMib TempRemMIB;
  e_IFX_LLDP_DEVICE_TYPE RemDeviceType;
	PRINT("\nIN->rx_do_rx_delete_info\n");
  lldp_port->rx.somethingChangedRemote = TRUE;
  /* As per IEEE 802.1AB 10.5.5.3 state diagram */
  if(lldp_port->rx.uiPresentMsap == TRUE) //coming from rxProcessFrame[shutdownframe]
  {
    PRINT("\nInvoking mibDeleteObjects for shutdown frame\n");
    /*Deleting My entry is done in HandleShutDownFrame*/
    /*Intimating LLDP Mgnr is done mibDeleteObjects()*/
    mibDeleteObjects(lldp_port); //msap is in currRecvmsap
    lldp_port->rx.uiTempIndex = -1; //would ve used the index.Reset it back.
  } 
  if(lldp_port->rx.uiTempIndex > -1)//coming from rxInfoAge=TRUE..Timer expired;
  {
    PRINT("\nfrom rxInfoAge=TRUE\n");
    int16 temp=lldp_port->rx.uiTempIndex;

		memcpy(&RemMacAddr.uiaMacAddress[0],&MsapTimer[temp]->Mac[0],6);
    getMibData(remoteMibMsap,lldp_port->uiPortIndex ,MsapTimer[temp]->Msap,(void *)&TempRemMIB);
			  
  	if(TempRemMIB.med_cap.deviceType == 4)
        RemDeviceType=IFX_LLDP_NCD;
      else if(TempRemMIB.med_cap.deviceType == 0)
        RemDeviceType=-1;
   		else 
        RemDeviceType=IFX_LLDP_EPD;

	  /*delete MIB data*/
    if(deleteMibData(lldp_port->uiPortIndex,MsapTimer[temp]->Msap) == MIB_SUCCESS)
    {
      /*if success then only intimate LLDP Mgnr about the expiry*/      
    	IFX_LLDP_PACKET_EXPIRED(RemDeviceType,&RemMacAddr); 
    }
    else
      PRINT("\nMIB delete Fail\n");

    /*delete all data in my array*/
    free(MsapTimer[temp]->Msap); //free msap
    MsapTimer[temp]->Msap = NULL;
  
    free(MsapTimer[temp]->Mac); //free mac
    MsapTimer[temp]->Mac = NULL;
     
    free(MsapTimer[temp]); //free entry
    MsapTimer[temp] = NULL;

  }
  lldp_port->rx.uiPresentMsap = FALSE;  
  //nullifying of rx.currRecvmsap is done in wait_for_frame XXX
	PRINT("\nOUT->rx_do_rx_delete_info\n");
}

/******************************************************************
*  Function Name    : rx_do_rx_update_info
*  Description      : This function takes actions when RX FSM is in 
*                     UPDATE_INFO state.We come here only for new MSAPS
*		      and existing ones with changes.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void rx_do_rx_update_info(struct lldp_port *lldp_port)
{
  //LLDP_FUNCTION_DEBUG;
  PRINT("\n\nIN->rx_do_rx_update_info\n\n");
  x_IFX_LLDP_TimerCallBackParam xLLDPTimerCB;

  /* As per IEEE 802.1AB 10.5.5.3 state diagram */
  mibUpdateObjects(lldp_port); //this ll take care of replacing/adding new one
  lldp_port->rx.somethingChangedRemote = TRUE;
  
  /*store the msap in the array and pass its index as the Timer ID below*/ 
  
  /*if msap already there.use the same entry again*/
  uint8 temp = 0,TimeoutId=0;
  if(lldp_port->rx.uiPresentMsap == TRUE)
  {
    /*Use the exisitng array index only as Timer ID*/
    for(temp=0;temp<IFX_LLDP_MAX_REMOTE_MIB_PER_PORT;temp++)
    {
      if(MsapTimer[temp] != NULL)
      {
        if(strcmp((char8*)lldp_port->rx.curr_recvmsap,(char8*)MsapTimer[temp]->Msap) == 0)
        {
          break;
        }
      }
    }
    TimeoutId = temp;//got the index
    /*Timer would ve been started for this.So stop this n start newly XXX*/
    PRINT("\nStopping timer---id %d \n",MsapTimer[temp]->uiTimerID); 
    IFX_LLDP_StopTimer(MsapTimer[temp]->uiTimerID);
  }
  else//new one
  {
    for(temp=0;temp<IFX_LLDP_MAX_REMOTE_MIB_PER_PORT;temp++)
    {
      if(MsapTimer[temp] == NULL)
      {
        MsapTimer[temp]=(struct lldp_rx_msaptimermap*)malloc(sizeof(struct lldp_rx_msaptimermap));
        TimeoutId = temp; //use array index as Timeout ID 
        /*store msap*/
        MsapTimer[temp]->Msap=malloc(lldp_port->rx.curr_msaplength);
        if(MsapTimer[temp]->Msap != NULL)
          memcpy(MsapTimer[temp]->Msap,lldp_port->rx.curr_recvmsap,lldp_port->rx.curr_msaplength);
        /*store mac.Required to intimate LLDP magnr when timer expires*/
        MsapTimer[temp]->Mac=(uint8*)malloc(6);
        if(MsapTimer[temp]->Mac != NULL)
          memcpy(&(MsapTimer[temp]->Mac[0]),&(lldp_port->rx.curr_recvmac[0]),6);
        break;
      }
    }
  }
  /*start timer for rxTTL XXX*/
  xLLDPTimerCB.unIndex = TimeoutId;
  xLLDPTimerCB.ucPort=lldp_port->uiPortIndex; 
  xLLDPTimerCB.uiMsgType = LLDP_REMOTE_TIMER_MSG;

  IFX_LLDP_StartTimer(lldp_port->rx.timers.rxTTL * 1000000,&xLLDPTimerCB,&(MsapTimer[temp]->uiTimerID));
  PRINT("\nStarted timer---id %d,index %d \n",MsapTimer[temp]->uiTimerID,temp); 
  PRINT("\nRX timer started with value: %d\n", lldp_port->rx.timers.rxTTL * 1000);
  //nullifying of rx.currRecvmsap is done in wait_for_frame XXX
  lldp_port->rx.uiPresentMsap = FALSE;
}
