/**************************************************************************
 **   FILE NAME       : ifx_lldp_cm.h
 **   PROJECT         : LLDP Config Module
 **   MODULES         : Config Module
 **   SRC VERSION     : V0.1
 **   DATE            : 29-3-2007
 **   AUTHOR          : Awanish Gautam,Supreet M S, Pavan G M.
 **   DESCRIPTION     : 
 **   DEPENDENCIES    : ifx_common_defs.h and ifx_lldp_defs.h
 **   FUNCTIONS       :
 **   COMPILER        : gcc
 **   REFERENCE       : Coding guide lines for VSS ,
 **
 **   COPYRIGHT       : Copyright  2004 Infineon Technologies AG
 **                     St. Martin Strasse 53; 81669 Mnchen, Germany
 **                     Any use of this Software is subject to the conclusion
 **                     of a respective License Agreement.Without such a
 **                     License Agreement no rights to the Software are granted.
 **  Version Control Section  **
 **   $Author$
 **   $Date$
 **   $Revisions$
 **   $Log$       Revision history
***********************************************************************/
#ifndef __IFX_LLDP_CM_H__
#define __IFX_LLDP_CM_H__

/* Constants */

/* Max Error Codes */
#define IFX_CM_MAX_ERROR_CODES 20

/* Tags to access deafult settings */
#define IFX_CM_FILE_RCVOIP_CONF  "/etc/lldpd.conf"

#define IFX_LLDP_FIFO "/tmp/lldpfifo"
/* Begin InfoTypes */
typedef enum
{
   /* Voip System Info type */
   IFX_CM_LLDP_CONFIG_CFG = 0,
   IFX_CM_TLV_SELECTION_CONFIG_CFG,
   IFX_CM_LLDP_STATS_CONFIG_CFG,
   IFX_CM_LOCAL_MIB_CFG, 
   IFX_CM_REMOTE_MIB_CFG_MSAP, 
   IFX_CM_REMOTE_MIB_CFG_MAC, 
   IFX_CM_LLDP_RAW_FRAME,
   IFX_CM_REMOTE_MAC,
   IFX_CM_LLDP_MISCELLANEOUS_CFG,
   IFX_CM_MAX_INFOTYPES /* 6 */
} e_IFX_CM_InfoType;

/* ErrorCodes */
typedef enum
{
   IFX_CM_FAIL = -1,
   IFX_CM_SUCCESS = 0,
   IFX_CM_NOSP_HANDLING
} e_IFX_CM_RespType;

/*this indicates the type of event occured.CM msges ve to filled based on INFO TYPE received
 *i.e local mib change or config changes*/

 enum e_LLdp_Event 
 {
   LLDP_MODULE_INIT_MSG = 70 ,
   LLDP_INIT_MSG ,
   LLDP_PORT_ENABLED_MSG,
   LLDP_PORT_DISABLED_MSG,
   LLDP_REMOTE_TIMER_MSG ,
   LLDP_LOCAL_TIMER_MSG ,
   LLDP_CONFIG_MSG ,
   LLDP_LOCAL_MIB_CHANGE_MSG,
   LLDP_REMOTE_FRAME_MSG,
   LLDP_SHUTDOWN_MSG
};
/*
typedef enum
{
   IFX_CM_LLDP_MODULE_INIT = 0,
   IFX_CM_LLDP_PORT_INIT,
   IFX_CM_LLDP_TLV_ADD,
   IFX_CM_LLDP_TLV_CHANGE,
   IFX_CM_LLDP_REMOTE_DB_EXTEND,
   IFX_CM_LLDP_REMOTE_DB_ENTRY_DEL,
//   IFX_CM_LLDP_REMOTE_TIMEOUT,
//   IFX_CM_LLDP_LOCAL_TIMEOUT,
   IFX_CM_LLDP_ENABLE,
   IFX_CM_LLDP_DISABLE,
   IFX_CM_LLDP_SHUTDOWN,
   //LLDP manager notify messages 
   IFX_CM_LLDP_NOTIFY_PACKET_RECEIVED,
   IFX_CM_LLDP_NOTIFY_PACKET_EXPIRED,
   
   IFX_CM_MAX_MSGTYPES
}e_IFX_CM_MsgType;
*/

typedef struct  
{
 enum e_LLdp_Event eEventType;
 uint16 EventID;  
 uchar8 ucPort;
}x_IFX_LLDP_Lldp_Event;
/* Begin - CmMsg */
/*
typedef union
{
  //x_IFX_LLDP_MODULE_INIT xModuleInit;
  //x_IFX_LLDP_PORT_INIT xPortInit;
 // x_IFX_LLDP_TLV_ADD xTlvAdd;
 // x_IFX_LLDP_TLV_CHANGE xTlvChange;
 // x_IFX_LLDP_REMOTE_DB_EXTEND xRemoteDbExtend;
 // x_IFX_LLDP_REMOTE_DB_ENTRY_DEL xRemoteDbDel;
  lldp_event  xlldpAgentMsg;
} ux_IFX_CM_DataType;
*/
typedef enum {
  IFX_LLDP_TIMER =1,
  IFX_LLDP_LLDP_MANAGER
}e_IFX_LLDP_MODULE_TYPE;

typedef struct
{
  e_IFX_LLDP_MODULE_TYPE eModType;
  char IfrName[10];
  uchar8 PortIndex;
  boolean bLldpduStorage;
  /* Data part of the message */
  x_IFX_LLDP_Lldp_Event xlldpAgentMsg; 
} x_IFX_CM_Msg;

/* PUBLIC Functions */

/* This function initializes the CM module. It should be invoked by all modules
   using CM fucntionalities */
PUBLIC char8 IFX_LLDP_CM_Init();

//PUBLIC char8 IFX_LLDP_CM_Shut(void);

/* This function initializes the shared memory and semaphores. It should be 
   called by all Process utilizing CM */
PUBLIC char8 IFX_LLDP_CM_SHM_Init(uchar8 ucNumPorts);

/* This function releases the created shared memory and semaphores */
PUBLIC char8 IFX_LLDP_CM_SHM_Shut(void);

/* This function gets an individual MIB for a given MAC address */
PUBLIC char8 IFX_LLDP_CM_GetCfgData(IN uchar8 ucInterfaceId,
                               IN uchar8 ucInfoType,
					           IN uchar8 *pucMac,
                               OUT void *pxData);

/* function sets an individual MIB in shared memory for a given MAC address */
PUBLIC char8 IFX_LLDP_CM_SetCfgData(IN uchar8 ucInterfaceId,
                               IN uchar8 ucInfoType,
					           IN uchar8 *pucMac,
                               IN void *pxData);

/* This function deletes an individual MIB in shared memory for a given MAC
   address */
PUBLIC char8 IFX_LLDP_CM_DeleteCfgData(IN uchar8 ucInterfaceId,
                                       IN uchar8 ucInfoType,
                                  IN uchar8 *pucMac);

/* This function returns the pointer to the data base of a particular 
   Info type */
PUBLIC char8 IFX_LLDP_CM_GetCfgPtr(IN uchar8 ucInterfaceId,
                              IN uchar8 ucInfoType,
                              OUT void **ppxDataPtr);

/* This function locks the shared memory through semaphore */
PUBLIC char8 IFX_LLDP_CM_LockCfg(void);

/* This function unlocks the shared memory through semaphore */
PUBLIC char8 IFX_LLDP_CM_UnLockCfg(void);

/* This function reads from the LLDP Agent FIFO */
PUBLIC char8 IFX_LLDP_CMFIFO_Read(void *Buffer, uint32 uiSize);

/* This function writes into the LLDP Agent FIFO */
PUBLIC char8 IFX_LLDP_CMFIFO_Write(void *Buffer, uint32 uiSize);

#endif /* __IFX_LLDP_CM_H__ */
