/**************************************************************************
 **   FILE NAME       : ifx_lldp_intf_defs.h
 **   PROJECT         : LLDP
 **   MODULES         : Interface Module
 **   SRC VERSION     : V0.1
 **   DATE            : 17-4-2007
 **   AUTHOR          : Awanish Gautam, Supreet M S, Pavan GM
 **   DESCRIPTION     : 
 **   DEPENDENCIES    : ifx_common_defs.h
 **   FUNCTIONS       :
 **   COMPILER        : gcc
 **   REFERENCE       : Coding guide lines for VSS ,
 **
 **   COPYRIGHT       : Copyright © 2004 Infineon Technologies AG
 **                     St. Martin Strasse 53; 81669 München, Germany
 **                     Any use of this Software is subject to the conclusion
 **                     of a respective License Agreement.Without such a
 **                     License Agreement no rights to the Software are granted.
 **  Version Control Section  **
 **   $Author$
 **   $Date$
 **   $Revisions$
 **   $Log$       Revision history
***********************************************************************/
#ifndef __IFX_LLDP_COMMON_INTF_DEFS_H__
#define __IFX_LLDP_COMMON_INTF_DEFS_H__

/* Defines all types of LLDP-MED devices */
typedef enum
{
  IFX_LLDP_NCD = 0,
  IFX_LLDP_EPD = 1
}e_IFX_LLDP_DEVICE_TYPE; 

/* This structure contains an IP address */
typedef struct {
  uint8 uiaIpAddress[4];
} x_IFX_LLDP_IP;

/* This structure contains a MAC address */
typedef struct {
  uint8 uiaMacAddress[6];
} x_IFX_LLDP_MAC;

/* Indicates the received packet status */
typedef enum {
  /* Indicates that a fresh new packet has been received */
  IFX_LLDP_PACKET_NEW =0,
  /* Indicates that an existing remote packet has been updated */
  IFX_LLDP_PACKET_EXISTING_CHANGED,
  /* Indicates that an TTL update is received for an existing remote entry */
  IFX_LLDP_PACKET_EXISTING_UNCHANGED
}e_IFX_LLDP_PACKET_STATUS;

/* Defines all possible MIB status information */
typedef enum {
  /* Indicates that still free memory is available in MIB */
  IFX_LLDP_MIB_FREE =0,
  /* Indicates that MIB memory is full */
  IFX_LLDP_MIB_FULL
}e_IFX_LLDP_MIB_STATUS;

/* This structure contains information related to a received LLDP packet. 
   This is sent out by LLDP Agent whenever a new remote LLDP packet is 
   received. */
typedef struct {
  /* Device Type - PC, LAN */
  e_IFX_LLDP_DEVICE_TYPE eDeviceType;
  /* Source MAC address */
  x_IFX_LLDP_MAC xMacAddress;
  /* Status of the packet - new, changed, unchanged */
  e_IFX_LLDP_PACKET_STATUS ePktStat;
  /* MIB Status - Free, Full */
  e_IFX_LLDP_MIB_STATUS eMibStatus;  
}x_IFX_LLDP_PACKET;

/* Contains information of a expired packet */
typedef struct {
  /* Device Type - PC, LAN */
  e_IFX_LLDP_DEVICE_TYPE eDeviceType;
  /* Source MAC address */
  x_IFX_LLDP_MAC xMacAddress;
} x_IFX_LLDP_PACKET_EXPIRED;

/* Message types which are sent by LLDP Agent */
typedef enum {
  /* Packet received message */
  IFX_LLDP_PACKET_RXVD_MSG = 0,
  /* Packet expired message */
  IFX_LLDP_PACKET_EXPIRED_MSG
} e_IFX_LLDP_MsgType;

/* Defines the Message sent by LLDP Agent to LLDP Manager */
typedef struct {
  /* Type of message */
  e_IFX_LLDP_MsgType eMsgType;

  /* Union of possible data sent */
  union {
    x_IFX_LLDP_PACKET xPacket;
    x_IFX_LLDP_PACKET_EXPIRED xPacketExpInfo;
  }ux_Data;
} x_IFX_LLDP_LLDPAGENT_MSG;

#endif /* __IFX_LLDP_COMMON_INTF_DEFS_H__ */
