/**************************************************************************
**   FILE NAME      : ifx_lldp_manager.c
**   PROJECT	    : Inca-Ip
**   MODULES        : LLDP
**   SRC VERSION    : V1.0
**
**   DATE           : 12-04-2007
**   AUTHOR         : Awanish G,Supreet MS, Pavan GM
**   DESCRIPTION    : LLDP Manager Implementation
**   COMPILER       :GCC
**   REFERENCE      :Coding Guideline Version 2.0
**							
**   COPYRIGHT      :Copyright © 20047
**                   Infineon Technologies AG
**                   St. Martin Strasse 53; 81669 München, Germany
**   DESCLAIMER     :Any use of this Software is subject to the conclusion 
**                   of a respective License Agreement.
**                   Without such a License Agreement no rights to the 
**         			 Software are granted. 
**
**  Version Control Section 
**   $Author$
**   $Date$
**   $Revisions$
**   $Log$       Revision history
***********************************************************************/

#include "ifx_common_defs.h"
#include "ifx_lldp_config.h"
#include "ifx_lldp_intf_defs.h"
#include "ifx_os.h"
#include "ifx_lldp_intf.h"
#include "ifx_lldp_manager.h"

/*****************************************************************************
*  Function Name        : main
*  Description          : This is this mail entry function to the LLDP 
*                         Manager process.
*                           
*  Input Values         : None 
*  Output Values        : None
*  Return Value         : 0
*  Notes                :
*****************************************************************************/
int32 main()
{
  int32 iChoice = -1, iModInitDone = 0, iPortInitDone = 0;
  x_IFX_LLDP_MODULE_INIT xInit;
  uchar8 IntfName[10]="eth0";
  e_IFX_LLDP_PORT_NAME ePort=IFX_LLDP_PC;
  x_IFX_LLDP_INIT_TLV_VALUES xTlvValues;
  x_IFX_LLDP_TLV_SUPPORT xTlvSupport;

#if 0
  /* Create a thread for receiving LLDP Agent messages */
  pthread_t threadId;
  if (pthread_create(&threadId, NULL, (void*)IFX_LLDP_WaitForMsg, NULL) != 0)
  {
    fprintf(stdout, "\nThread creation failed. Exiting...\n");
    return 0;
  }
#endif

  /* Take user inputs in an infinite loop */
  while(1)
  {
    fprintf(stdout, "\nLLDP MANAGER MENU\n");
    fprintf(stdout, "-----------------\n");
    fprintf(stdout, "1  - Module Init\n");
    fprintf(stdout, "2  - Port Init\n");
    fprintf(stdout, "3  - Enable Port\n");
    fprintf(stdout, "4  - Disable Port\n");
    fprintf(stdout, "5  - Change Local Config\n");
    fprintf(stdout, "6  - Add TLV\n");
    fprintf(stdout, "7  - Delete TLV\n");
    fprintf(stdout, "8  - Modify TLV\n");
    fprintf(stdout, "9  - Extend MIB\n");
    fprintf(stdout, "10 - Delete Rx MIB Entry\n");
    fprintf(stdout, "11 - Request Port Statistics\n");
    fprintf(stdout, "12 - Request Remote PDU\n");
    fprintf(stdout, "13 - Request Remote MIB Data\n");
    fprintf(stdout, "14 - Shutdown\n\n");

    fprintf(stdout, "Enter Choice:");
    if ((iChoice = IFX_LLDP_GetData()) < 0)
      /* If 'Enter' key is pressed, then continue */
      continue;

    /* Handle menu selection */
    switch(iChoice)
    {
      case 1:
      {
        /* Initialize LLDP Agent */
        xInit.uiOpMode = IFX_LLDP_EPD;

        /* Register the callback function */
        xInit.pFuncPtr = IFX_LLDP_WaitForMsg;

        fprintf(stdout, "\nNumber of ports [dafault = 2]:");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xInit.ucNumPorts = IFX_LLDP_MAX_PORTS;
        else
          xInit.ucNumPorts = (uchar8)iChoice;

        fprintf(stdout, "\nMIB entries to be supported [default = 10]:");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xInit.uiSupportedMibEntries = 10;
        else
          xInit.uiSupportedMibEntries = (uint32)iChoice;
       
        fprintf(stdout, "\nStore PDU (0 - No, 1 - Yes) [default = 0]: ");
        if ((xInit.bLldpduStorage = IFX_LLDP_GetData()) < 0)
          xInit.bLldpduStorage = FALSE;

        if (IFX_LLDP_MODULE_INIT(&xInit) < 0)
        {
          fprintf(stdout, "\nModule Init failed. Exiting...\n");
          return 0;
        }
        iModInitDone = 1;
        break;
      }
      case 2:
      {
        /* Initialising port */
        x_IFX_LLDP_CONFIGCFG xCfg;

        /* Supported TLVs */
        xTlvSupport.bChassisId = 1;
        xTlvSupport.bPortId = 1;
        xTlvSupport.bSystemCapabilities=1;
        xTlvSupport.bTimeToLive=1;
        xTlvSupport.bMacPhyConfigurationStatus=1;
        xTlvSupport.bLldpMedCapabilities=1;
        xTlvSupport.bNetworkPolicy=1;
        xTlvSupport.bExtendedPowerViaMdi=1;
        xTlvSupport.bEndOfLldpdu=1;
        xTlvSupport.bPortVlanId=0;
        xTlvSupport.bPortAndProtocolVlanId=0;
        xTlvSupport.bVlanName=0;
        xTlvSupport.bProtocolIdentity=0;

        /* Initialise TLV values */

        /* Chassis Id TLV */
        memset(&(xTlvValues.ucChassisId), '\0', sizeof(xTlvValues.ucChassisId));
        xTlvValues.ucChassisId[0]=0xad;
        xTlvValues.ucChassisId[1]=0x14;
        xTlvValues.ucChassisId[2]=0x16;
        xTlvValues.ucChassisId[3]=0x28;

        /* Port Id TLV */
        memset(&(xTlvValues.ucPortId), '\0', sizeof(xTlvValues.ucPortId));
        xTlvValues.ucPortId[0]=0x08;
        xTlvValues.ucPortId[1]=0x02;
        xTlvValues.ucPortId[2]=0x06;
        xTlvValues.ucPortId[3]=0x2b;
        xTlvValues.ucPortId[4]=0x97;
        xTlvValues.ucPortId[5]=0xfe;
        xTlvValues.ucPortId[6]='\0';
        /* System Capabilities TLV */
        memset(&(xTlvValues.xSystemCapabilities),0,
          sizeof(x_IFX_LLDP_SYS_CAPABILITIES));
        xTlvValues.xSystemCapabilities.bTelephone=1;
        xTlvValues.xSystemCapabilities.bTelephone_status=1;

        /* MAC/PHY Configuration TLV */
        xTlvValues.xMacPhyConfigurationStatus.uiSupport=1;
        xTlvValues.xMacPhyConfigurationStatus.uiStatus=1;
        xTlvValues.xMacPhyConfigurationStatus.uiAdvertisedCapabilities=10;

        /* LLDP MED Capabilities TLV */
        memset(&(xTlvValues.xLldpMedCapabilities),0,
          sizeof(x_IFX_LLDP_MED_CAPABILITIES));
        xTlvValues.xLldpMedCapabilities.bCapabilities=1;
        xTlvValues.xLldpMedCapabilities.bNetworkPolicy=1;
        xTlvValues.xLldpMedCapabilities.bExtendedPowerMdiPD=1;
        xTlvValues.xLldpMedCapabilities.uiDeviceType=IFX_LLDP_INTF_CLASSIII;

        /* Network Policy TLV */
        memset(&(xTlvValues.xNetworkPolicy),0,
          sizeof(x_IFX_LLDP_NETWORK_POLICY));
        xTlvValues.xNetworkPolicy.uiNoOfNetworkPolicyTlvs = 1;
        xTlvValues.xNetworkPolicy.xNetworkPolicyArray[0].uiApplicationType=1;
        xTlvValues.xNetworkPolicy.xNetworkPolicyArray[0].bUnknownFlag=1;
        xTlvValues.xNetworkPolicy.xNetworkPolicyArray[0].bTaggedFlag=1;
        xTlvValues.xNetworkPolicy.xNetworkPolicyArray[0].uiVlanId=1;
        xTlvValues.xNetworkPolicy.xNetworkPolicyArray[0].uiL2Priority=1;
        xTlvValues.xNetworkPolicy.xNetworkPolicyArray[0].uiDscpValue=1;

        /* Extended Power Via MDI TLV */
        xTlvValues.xExtendedPowerViaMdi.uiType=1;
        xTlvValues.xExtendedPowerViaMdi.uiSource=1;
        xTlvValues.xExtendedPowerViaMdi.uiPriority=1;
        xTlvValues.xExtendedPowerViaMdi.uiValue=25;

        /* Initialize local config values */
        xCfg.ucIntfAdminStatus = IFX_LLDP_INTF_ENABLED_RXTX;
        xCfg.unIntfReinitDelay = 2;
        xCfg.unIntfMsgTxHold = 4;
        xCfg.unIntfMsgTxInterval = 30;
        xCfg.unIntfTxDelay = 2;
        xCfg.uiIntfMedFastStartRepeatCount = 0;

        if (iModInitDone == 1)
        {
          /* Enabling port */
          uint32 uiPort;
          fprintf(stdout, "\nEnter Port Id (0 - PC, 1 - LAN): ");
          scanf("%d", &uiPort);
          if ((uiPort != IFX_LLDP_PC) && (uiPort != IFX_LLDP_LAN))
            fprintf(stdout, "\nInvalid Port");
          else
          {
            ePort = uiPort;
            if (ePort == IFX_LLDP_PC)
              xTlvValues.xMacPhyConfigurationStatus.uiMauType = IFX_LLDP_MAU_TYPE_PC;
            else
              xTlvValues.xMacPhyConfigurationStatus.uiMauType = IFX_LLDP_MAU_TYPE_LAN;

            if (IFX_LLDP_PORT_INIT(IntfName, ePort,
                &xCfg, xTlvSupport,&xTlvValues) < 0)
              fprintf(stdout, "Port Init failed");
            else
              iPortInitDone = 1;
          }
        }
        break;
      }
      case 3:
      {
        /* Enabling port */
        uint32 uiPort;
        fprintf(stdout, "\nEnter Port Id (0 - PC, 1 - LAN): ");
        scanf("%d", &uiPort);
        if ((uiPort != IFX_LLDP_PC) && (uiPort != IFX_LLDP_LAN))
          fprintf(stdout, "\nInvalid Port");
        else
        {
          IFX_LLDP_ENABLE_PORT(uiPort);
        }
        break;
      }
      case 4:
      {
        /* Disabling port */
        if ((iModInitDone == 1) && (iPortInitDone == 1))
        {
          uint32 uiPort;
          fprintf(stdout, "\nEnter Port Id (0 - PC, 1 - LAN): ");
          scanf("%d", &uiPort);
          if ((uiPort != IFX_LLDP_PC) && (uiPort != IFX_LLDP_LAN))
            fprintf(stdout, "\nInvalid Port");
          else
          {
            IFX_LLDP_DISABLE_PORT(uiPort);
          }
        }  
        break;
      }
      case 5:
      {
        /* Change local config values */
        if ((iModInitDone == 1) && (iPortInitDone == 1))
        {
          uint32 uiPort;
          fprintf(stdout, "\nEnter Port Id (0 - PC, 1 - LAN): ");
          //scanf("%d", &uiPort);
          uiPort = IFX_LLDP_GetData();
          if ((uiPort != IFX_LLDP_PC) && (uiPort != IFX_LLDP_LAN))
          {
            fprintf(stdout, "\nInvalid Port");
            continue;
          }
          fflush(stdin);
          fflush(stdout);
          fprintf(stdout, "Admin Status(0 - Disable, 1 - Tx, 2 - Rx, 3 - TxRx): ");
          iChoice = IFX_LLDP_GetData();
          x_IFX_LLDP_CONFIGCFG xCfg;

          int32 iTimerVal;
          fprintf(stdout, "\nReinitDelay (in secs) [default = 2]: ");
          if ((iTimerVal = IFX_LLDP_GetData()) < 0)
            xCfg.unIntfReinitDelay = 2;
          else
            xCfg.unIntfReinitDelay = iTimerVal;
          fprintf(stdout, "\nMsgTxHold (in secs) [default = 4]: ");
          if ((iTimerVal = IFX_LLDP_GetData()) < 0)
            xCfg.unIntfMsgTxHold = 4;
          else
            xCfg.unIntfMsgTxHold = iTimerVal;
          fprintf(stdout, "\nMsgTxInterval (in secs) [default = 30]: ");
          if ((iTimerVal = IFX_LLDP_GetData()) < 0)
            xCfg.unIntfMsgTxInterval = 30;
          else
            xCfg.unIntfMsgTxInterval = iTimerVal;
          fprintf(stdout, "\nTxDelay (in secs) [default = 2]: ");
          if ((iTimerVal = IFX_LLDP_GetData()) < 0)
            xCfg.unIntfTxDelay = 2;
          else
            xCfg.unIntfTxDelay = iTimerVal;
          fprintf(stdout, "\nMedFastStartRepeatCount [default = 0]: ");
          if ((iTimerVal = IFX_LLDP_GetData()) < 0)
            xCfg.uiIntfMedFastStartRepeatCount = 0;
          else
            xCfg.uiIntfMedFastStartRepeatCount = iTimerVal;

          /* Update configuration using interface APIs */
          switch(iChoice)
          {
            case -1:
            {
              xCfg.ucIntfAdminStatus = IFX_LLDP_INTF_ENABLED_RXTX; 
              IFX_LLDP_LOCAL_CONFIG_UPDATED(uiPort, &xCfg);
              break;
            }
            case 0:
            {
              xCfg.ucIntfAdminStatus = IFX_LLDP_INTF_DISABLED; 
              IFX_LLDP_LOCAL_CONFIG_UPDATED(uiPort, &xCfg);
              break;
            }
            case 1:
            {
              xCfg.ucIntfAdminStatus = IFX_LLDP_INTF_ENABLED_TX_ONLY; 
              IFX_LLDP_LOCAL_CONFIG_UPDATED(uiPort, &xCfg);
              break;
            }
            case 2:
            {
              xCfg.ucIntfAdminStatus = IFX_LLDP_INTF_ENABLED_RX_ONLY; 
              IFX_LLDP_LOCAL_CONFIG_UPDATED(uiPort, &xCfg);
              break;
            }
            case 3:
            {
              xCfg.ucIntfAdminStatus = IFX_LLDP_INTF_ENABLED_RXTX; 
              IFX_LLDP_LOCAL_CONFIG_UPDATED(uiPort, &xCfg);
              break;
            }
            default:
              break;
          }
        }
        break;
      }

      case 6:
      {
        /* Add a TLV */
        if ((iModInitDone == 1) && (iPortInitDone == 1))
          IFX_LLDP_HandleAddTlvSelection(TRUE);
        break;
      }
      case 7:
      {
        /* Delete a TLV */
        if ((iModInitDone == 1) && (iPortInitDone == 1))
          IFX_LLDP_HandleDeleteTlvSelection();
        break;
      }
      case 8:
      {
        /* Modify an existing TLV */
        if ((iModInitDone == 1) && (iPortInitDone == 1))
          IFX_LLDP_HandleAddTlvSelection(FALSE);
        break;
      }
      case 9:
      {
        /* Extending remote database */
        if ((iModInitDone == 1) && (iPortInitDone == 1))
        {
          fprintf(stdout, "\nNumber of elements to be added: ");  
          scanf("%d", &iChoice);
          IFX_LLDP_REMOTE_DB_EXTEND(iChoice);
        }
        break;
      }
      case 10:
      {
        /* Delete an entry from remote MIB */
        if ((iModInitDone == 1) && (iPortInitDone == 1))
        {
          uint8 ucaArr[6];
          uint32 uiaArr[6];
          int32 i;
          for (i = 0; i < 6; i++)
          {
            fprintf(stdout, "\nEnter %dth element of MAC address (in HEX):", i);
            scanf("%x", &uiaArr[i]);
            ucaArr[i] = (uint8)uiaArr[i];
          }
          x_IFX_LLDP_MAC xMacAddr;
          memcpy(xMacAddr.uiaMacAddress, ucaArr, 6);
          fprintf(stdout, "MAC Address Entered: %x:%x:%x:%x:%x:%x\n",
            xMacAddr.uiaMacAddress[0],
            xMacAddr.uiaMacAddress[1],
            xMacAddr.uiaMacAddress[2],
            xMacAddr.uiaMacAddress[3],
            xMacAddr.uiaMacAddress[4],
            xMacAddr.uiaMacAddress[5]);

          if (IFX_LLDP_REMOTE_DB_ENTRY_DEL(xMacAddr) == 0)
            fprintf(stdout, "Remote MIB entry deleted successfully\n");
          else
            fprintf(stdout, "Remote MIB entry deletion failed\n");
        }
        break;
      }
      case 11:
      {
        /* Fetch and display port stats */
        if ((iModInitDone == 1) && (iPortInitDone == 1))
          IFX_LLDP_DisplayPortStats();
        break;
      }
      case 12:
      {
        /* Fetch and display a remote PDU */
        if ((iModInitDone == 1) && (iPortInitDone == 1))
          IFX_LLDP_GetRemoteMibPdu();
        break;
      }

      case 13:
      {
        /* Fetch and display remote MIB data */
        if ((iModInitDone == 1) && (iPortInitDone == 1))
          IFX_LLDP_GetRemoteMibData();
        break;
      }
      case 14:
      {
        /* Safely shutdown LLDP Agent and Manager */
        IFX_LLDP_SHUTDOWN();
        return 0;
      }
      default:
        break;
    } /* End of switch */
  } /* End of while */
}

/*****************************************************************************
*  Function Name        : IFX_LLDP_HandleAddTlvSelectio
*  Description          : This function handles the "Add TLV" menu selection.
*                         It takes user input for the selected TLV and updates
*                         it in LLDPD.
*                           
*  Input Values         : bAdd - If TRUE, Add a TLV; if FALSE, modify the TLV. 
*  Output Values        : None
*  Return Value         : 0
*  Notes                :
*****************************************************************************/
void IFX_LLDP_HandleAddTlvSelection(boolean bAdd)
{
  int32 iChoice;
  boolean bContinue = 1;
  x_IFX_LLDP_TLV_VALUES xNewTlv;
  uint32 uiPort;
  fprintf(stdout, "\nEnter Port Id (0 - PC, 1 - LAN): ");
  scanf("%d", &uiPort);
  if ((uiPort != IFX_LLDP_PC) && (uiPort != IFX_LLDP_LAN))
  {
    fprintf(stdout, "\nInvalid Port");
    return;
  }

  while(bContinue)
  {
    fprintf(stdout, "\nTLV Menu\n");
    fprintf(stdout, "--------\n");
    fprintf(stdout, "1  - ChassisId\n");
    fprintf(stdout, "2  - PortId\n");
    fprintf(stdout, "3  - System Capabilities\n");
    fprintf(stdout, "4  - MacPhy Config Status\n");
    fprintf(stdout, "5  - MED Capabilities\n");
    fprintf(stdout, "6  - Network Policy\n");
    fprintf(stdout, "7  - Extended Power Via MDI\n");

    fprintf(stdout, "\nSelect TLV Type: ");
    if ((iChoice = IFX_LLDP_GetData()) < 0)
    {
      bContinue = 1;
      continue;
    }
    else
      bContinue = 0;

    switch(iChoice)
    {
      case 1:
      {
        uint32 uiaArr[6];
        uint32 i = 0;
        for (i = 0; i < 4; i++)
        {
          fprintf(stdout, "\nEnter %d element of IP Address in HEX: ", i);
          scanf("%x", &uiaArr[i]);
          xNewTlv.xValues.ucChassisId[i] = (uint8)uiaArr[i];
        }
        fprintf(stdout, "Chassis Id: %x:%x:%x:%x\n",
          xNewTlv.xValues.ucChassisId[0],
          xNewTlv.xValues.ucChassisId[1],
          xNewTlv.xValues.ucChassisId[2],
          xNewTlv.xValues.ucChassisId[3]);

        xNewTlv.uiNewTlv = IFX_LLDP_INTF_CHASSIS_ID;
        break;
      }
      case 2:
      {
        uint32 uiaArr[6];
        uint32 i = 0;
        for (i = 0; i < 6; i++)
        {
          fprintf(stdout, "\nEnter %d element of MAC Address in HEX: ", i);
          scanf("%x", &uiaArr[i]);
          xNewTlv.xValues.ucPortId[i] = (uint8)uiaArr[i];
        }
        fprintf(stdout, "Port Id: %x:%x:%x:%x\n",
          xNewTlv.xValues.ucPortId[0],
          xNewTlv.xValues.ucPortId[1],
          xNewTlv.xValues.ucPortId[2],
          xNewTlv.xValues.ucPortId[3]);

        xNewTlv.uiNewTlv = IFX_LLDP_INTF_PORT_ID;
        break;
      }
      case 3:
      {
        xNewTlv.uiNewTlv = IFX_LLDP_INTF_SYSTEM_CAPABILITIES;
        fprintf(stdout, "\nOther (0 - Not Supported, 1 - Supported) [default = 0]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bOther = 0;
        else
          xNewTlv.xValues.xSystemCapabilities.bOther = iChoice;
        fprintf(stdout, "\nOther Status (0 - Disable, 1 - Enable) [default = 0]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bOther = 0;
        else
          xNewTlv.xValues.xSystemCapabilities.bOther_status = iChoice;
        fprintf(stdout, "\nRepeater (0 - Not Supported, 1 - Supported) [default = 0]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bRepeater = 0;
        else
          xNewTlv.xValues.xSystemCapabilities.bRepeater = iChoice;
        fprintf(stdout, "\nRepeater Status (0 - Disable, 1 - Enable) [default = 0]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bRepeater_status = 0;
       else
         xNewTlv.xValues.xSystemCapabilities.bRepeater_status = iChoice;
        fprintf(stdout, "\nBridge (0 - Not Supported, 1 - Supported) [default = 0]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bBridge = 0;
        else
          xNewTlv.xValues.xSystemCapabilities.bBridge = iChoice;
        fprintf(stdout, "\nBridge Status (0 - Disable, 1 - Enable) [default = 0]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bBridge_status = 0;
        else
          xNewTlv.xValues.xSystemCapabilities.bBridge_status = iChoice;
        fprintf(stdout, "\nWLAN App (0 - Not Supported, 1 - Supported) [default = 0]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bWlanAp = 0;
        else
          xNewTlv.xValues.xSystemCapabilities.bWlanAp = iChoice;
        fprintf(stdout, "\nWLAN App Status (0 - Disable, 1 - Enable) [default = 0]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bWlanAp_status = 0;
        else
          xNewTlv.xValues.xSystemCapabilities.bWlanAp_status = iChoice;
        fprintf(stdout, "\nRouter (0 - Not Supported, 1 - Supported) [default = 0]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bRouter = 0;
        else
          xNewTlv.xValues.xSystemCapabilities.bRouter = iChoice;
        fprintf(stdout, "\nRouter Status (0 - Disable, 1 - Enable) [default = 0]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bRouter_status = 0;
        else
          xNewTlv.xValues.xSystemCapabilities.bRouter_status = iChoice;

        fprintf(stdout, "\nTelephone (0 - Not Supported, 1 - Supported) [default = 1]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bTelephone = 1;
        else
          xNewTlv.xValues.xSystemCapabilities.bTelephone = iChoice;
        fprintf(stdout, "\nTelephone Status (0 - Disable, 1 - Enable) [default = 1]: )");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bTelephone_status = 1;
        else
          xNewTlv.xValues.xSystemCapabilities.bTelephone_status = iChoice;
        fprintf(stdout, "\nCable Device (0 - Not Supported, 1 - Supported) [default = 0]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bCableDevice = 0;
        else
          xNewTlv.xValues.xSystemCapabilities.bCableDevice = iChoice;
        fprintf(stdout, "\nCable Device Status (0 - Disable, 1 - Enable) [default = 0]: )");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bCableDevice_status = 0;
        else
          xNewTlv.xValues.xSystemCapabilities.bCableDevice_status = iChoice;
        fprintf(stdout, "\nStation Only (0 - Not Supported, 1 - Supported) [default = 0]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bStationOnly = 0;
        else
          xNewTlv.xValues.xSystemCapabilities.bStationOnly = iChoice;
        fprintf(stdout, "\nStation Only Status (0 - Disable, 1 - Enable) [default = 0]: )");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xSystemCapabilities.bStationOnly_status = 0;
        else
          xNewTlv.xValues.xSystemCapabilities.bStationOnly_status = iChoice;
        break;
      }
      case 4:
      {
        xNewTlv.uiNewTlv = IFX_LLDP_INTF_MAC_PHY_CONFIGURATION_STATUS;
        fprintf(stdout, "\nAuto-negotiation Support [default = 1]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xMacPhyConfigurationStatus.uiSupport = 1;
        else
          xNewTlv.xValues.xMacPhyConfigurationStatus.uiSupport = (uint8)iChoice;
        fprintf(stdout, "\nAuto-negotiation Status [default = 1]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xMacPhyConfigurationStatus.uiStatus = 1;
        else
          xNewTlv.xValues.xMacPhyConfigurationStatus.uiStatus = (uint8)iChoice;
        fprintf(stdout, "\nAdvertised Capabilities [default = 10]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xMacPhyConfigurationStatus.uiAdvertisedCapabilities = 10;
        else
          xNewTlv.xValues.xMacPhyConfigurationStatus.uiAdvertisedCapabilities = 
            (uint16)iChoice;
        fprintf(stdout, "\nMAU Type [default = 10]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xMacPhyConfigurationStatus.uiMauType = 10;
        else
          xNewTlv.xValues.xMacPhyConfigurationStatus.uiMauType = (uint16)iChoice;
        break;
      }
      case 5:
      {
        xNewTlv.uiNewTlv = IFX_LLDP_INTF_MED_CAPABILITIES;
        fprintf(stdout, "\nCapabilities (0 - Disable, 1 - Enable) [default = 1]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xLldpMedCapabilities.bCapabilities = 1;
        else
          xNewTlv.xValues.xLldpMedCapabilities.bCapabilities = iChoice;
        fprintf(stdout, "\nNetwork Policy (0 - Disable, 1 - Enable) [default = 1]: ");
        iChoice = IFX_LLDP_GetData();
        if (iChoice < 0)
          xNewTlv.xValues.xLldpMedCapabilities.bNetworkPolicy = 1;
        else
          xNewTlv.xValues.xLldpMedCapabilities.bNetworkPolicy = iChoice;
        fprintf(stdout, "\n\nn/w policy: %d\n\n", 
          xNewTlv.xValues.xLldpMedCapabilities.bNetworkPolicy);
        fprintf(stdout, "\nExtended Power-via-MDI (0 - Disable, 1 - Enable) [default = 1]: ");
        iChoice = IFX_LLDP_GetData();
        if (iChoice < 0)
          xNewTlv.xValues.xLldpMedCapabilities.bExtendedPowerMdiPD = 1;
        else
          xNewTlv.xValues.xLldpMedCapabilities.bExtendedPowerMdiPD = iChoice;

        fprintf(stdout, "\n\nextended power: %d\n\n", 
          xNewTlv.xValues.xLldpMedCapabilities.bExtendedPowerMdiPD);

        if (xNewTlv.xValues.xLldpMedCapabilities.bExtendedPowerMdiPD == 1)
            xNewTlv.xValues.xLldpMedCapabilities.bExtendedPowerMdiPse = 0;
        else
          xNewTlv.xValues.xLldpMedCapabilities.bExtendedPowerMdiPse = 0;

        fprintf(stdout, "\nLocation Identification (0 - Disable, 1 - Enable) [default = 0]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xLldpMedCapabilities.bLocationIdentification = 0;
        else
          xNewTlv.xValues.xLldpMedCapabilities.bLocationIdentification = iChoice;
        fprintf(stdout, "\nInventory (0 - Disable, 1 - Enable) [default = 0]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xLldpMedCapabilities.bInventory = 0;
        else
          xNewTlv.xValues.xLldpMedCapabilities.bInventory = iChoice;
        fprintf(stdout, "\nDevice Type (1 - EPDI, 2 - EPDII, 3  - EPDIII, 4 - NCD) [default = 3]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xLldpMedCapabilities.uiDeviceType = IFX_LLDP_INTF_CLASSIII;
        else
          xNewTlv.xValues.xLldpMedCapabilities.uiDeviceType = iChoice;

        break;
      }
      case 6:
      {
        xNewTlv.uiNewTlv = IFX_LLDP_INTF_NETWORK_POLICY;
        fprintf(stdout, "\nNumber of network Policy TLVs [default = 1]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xNetworkPolicy.uiNoOfNetworkPolicyTlvs = 1;
        else
          xNewTlv.xValues.xNetworkPolicy.uiNoOfNetworkPolicyTlvs = (uint8)iChoice;

        int32 i = 0;
        for (i = 0; i < xNewTlv.xValues.xNetworkPolicy.uiNoOfNetworkPolicyTlvs; i++)
        {
          fprintf(stdout, "\nApplication Type (0-255) [default = 1]: ");
          if ((iChoice = IFX_LLDP_GetData()) < 0)
            xNewTlv.xValues.xNetworkPolicy.xNetworkPolicyArray[i].uiApplicationType = 1;
          else
            xNewTlv.xValues.xNetworkPolicy.xNetworkPolicyArray[i].uiApplicationType = (uint8)iChoice;
          fprintf(stdout, "\nUnknown Flag (0 - FALSE, 1 - TRUE) [default = 1]: ");
          if ((iChoice = IFX_LLDP_GetData()) < 0)
            xNewTlv.xValues.xNetworkPolicy.xNetworkPolicyArray[i].bUnknownFlag = 1;
          else
            xNewTlv.xValues.xNetworkPolicy.xNetworkPolicyArray[i].bUnknownFlag = iChoice;
          fprintf(stdout, "\nTagged Flag (0 - Untagged, 1 - Tagged) [default = 0]: ");
          if ((iChoice = IFX_LLDP_GetData()) < 0)
            xNewTlv.xValues.xNetworkPolicy.xNetworkPolicyArray[i].bTaggedFlag = 0;
          else
            xNewTlv.xValues.xNetworkPolicy.xNetworkPolicyArray[i].bTaggedFlag = iChoice;

          if (xNewTlv.xValues.xNetworkPolicy.xNetworkPolicyArray[i].bUnknownFlag == FALSE)
          {
            if (xNewTlv.xValues.xNetworkPolicy.xNetworkPolicyArray[i].bTaggedFlag == TRUE)
            {
              fprintf(stdout, "\nVLAN Id (1-4094) [default = 1]: ");
              if ((iChoice = IFX_LLDP_GetData()) < 0)
                xNewTlv.xValues.xNetworkPolicy.xNetworkPolicyArray[i].uiVlanId = 1;
              else
                xNewTlv.xValues.xNetworkPolicy.xNetworkPolicyArray[i].uiVlanId = (uint16)iChoice;
              fprintf(stdout, "\nPriority (0-7) [default = 0]: ");
              if ((iChoice = IFX_LLDP_GetData()) < 0)
                xNewTlv.xValues.xNetworkPolicy.xNetworkPolicyArray[i].uiL2Priority = 0;
              else
                xNewTlv.xValues.xNetworkPolicy.xNetworkPolicyArray[i].uiL2Priority = (uint8)iChoice;
            }
            fprintf(stdout, "\nDSCP Value (0-63) [default = 0]: ");
            if ((iChoice = IFX_LLDP_GetData()) < 0)
              xNewTlv.xValues.xNetworkPolicy.xNetworkPolicyArray[i].uiDscpValue = 0;
            else
              xNewTlv.xValues.xNetworkPolicy.xNetworkPolicyArray[i].uiDscpValue = (uint8)iChoice;
          }
        }
        break;
      }
      case 7:
      {
        xNewTlv.uiNewTlv = IFX_LLDP_INTF_EXTENDED_POWER_VIA_MDI;
        fprintf(stdout, "\nType (0 - PSE, 1 - PD) [default = 1]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xExtendedPowerViaMdi.uiType = 1;
        else
          xNewTlv.xValues.xExtendedPowerViaMdi.uiType = (uint8)iChoice;
        fprintf(stdout, "\nPower Source (0-3) [default = 1]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xExtendedPowerViaMdi.uiSource = 1;
        else
          xNewTlv.xValues.xExtendedPowerViaMdi.uiSource = (uint8)iChoice;
        fprintf(stdout, "\nPower Priority (0-3) [default = 2]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xExtendedPowerViaMdi.uiPriority = 2;
        else
          xNewTlv.xValues.xExtendedPowerViaMdi.uiPriority = (uint8)iChoice;
        fprintf(stdout, "\nPower Value (0-1023) [default = 130]: ");
        if ((iChoice = IFX_LLDP_GetData()) < 0)
          xNewTlv.xValues.xExtendedPowerViaMdi.uiValue = 130;
        else
          xNewTlv.xValues.xExtendedPowerViaMdi.uiValue = (uint16)iChoice;

        break;
      }
      default:
      {
        bContinue = 1;
        fprintf(stdout, "Invalid input\n");
        break;
      }
    } /* End of switch */
  } /* End of while */

  if (bAdd)
  {
    if (IFX_LLDP_TLV_ADD(uiPort, &xNewTlv) == 0)
      fprintf(stdout, "TLV Added Successfully\n");
    else
      fprintf(stdout, "TLV Addition Failed\n");
  }
  else
  {
    if (IFX_LLDP_TLV_CHANGE(uiPort, &xNewTlv) == 0)
      fprintf(stdout, "TLV Modified Successfully\n");
    else
      fprintf(stdout, "TLV Modification Failed\n");
  }
}

/*****************************************************************************
*  Function Name        : IFX_LLDP_HandleDeleteTlvSelectio
*  Description          : This function handles the "Delete TLV" menu selection
*                           
*  Input Values         : None 
*  Output Values        : None
*  Return Value         : 0
*  Notes                :
*****************************************************************************/
void IFX_LLDP_HandleDeleteTlvSelection()
{
  int32 iChoice;
  boolean bContinue = 1;
  int32 iRet = 0;
  uint32 uiPort;
  fprintf(stdout, "\nEnter Port Id (0 - PC, 1 - LAN): ");
  scanf("%d", &uiPort);
  if ((uiPort != IFX_LLDP_PC) && (uiPort != IFX_LLDP_LAN))
  {
    fprintf(stdout, "\nInvalid Port");
    return;
  }

  while(bContinue)
  {
    fprintf(stdout, "\nTLV Menu\n");
    fprintf(stdout, "\n--------\n");
    fprintf(stdout, "\n1  - ChassisId\n");
    fprintf(stdout, "2  - PortId\n");
    fprintf(stdout, "3  - System Capabilities\n");
    fprintf(stdout, "4  - MacPhy Config Status\n");
    fprintf(stdout, "5  - MED Capabilities\n");
    fprintf(stdout, "6  - Network Policy\n");
    fprintf(stdout, "7  - Extended Power Via MDI\n");

    fprintf(stdout, "\nSelect TLV Type: ");  
    if ((iChoice = IFX_LLDP_GetData()) < 0)
    {
      bContinue = 1;
      continue;
    }
    else
      bContinue = 0;

    switch(iChoice)
    {
      case 1:
        iRet = IFX_LLDP_TLV_DEL (uiPort,IFX_LLDP_INTF_CHASSIS_ID);
        break;
      case 2:
        iRet = IFX_LLDP_TLV_DEL (uiPort,IFX_LLDP_INTF_PORT_ID);
        break;
      case 3:
        iRet = IFX_LLDP_TLV_DEL (uiPort,IFX_LLDP_INTF_SYSTEM_CAPABILITIES);
        break;
      case 4:
        iRet = IFX_LLDP_TLV_DEL (uiPort,IFX_LLDP_INTF_MAC_PHY_CONFIGURATION_STATUS);
        break;
      case 5:
        iRet = IFX_LLDP_TLV_DEL (uiPort,IFX_LLDP_INTF_MED_CAPABILITIES);
        break;
      case 6:
        iRet = IFX_LLDP_TLV_DEL (uiPort,IFX_LLDP_INTF_NETWORK_POLICY);
        break;
      case 7:
        iRet = IFX_LLDP_TLV_DEL (uiPort,IFX_LLDP_INTF_EXTENDED_POWER_VIA_MDI);
        break;
      default:
        fprintf(stdout, "\nInvalid input\n");
        bContinue = 1;
        break;
    } /* End of switch */
  } /* End of while */

  if (iRet == 0)
    fprintf(stdout, "TLV Deleted Successfully\n");
  else
    fprintf(stdout, "TLV Deletion failed\n");
}

/*****************************************************************************
*  Function Name        : IFX_LLDP_GetRemoteMibPdu
*  Description          : This function displays a remote LLDP PDU based upon
*                         the MAC address entered by the user.
*                           
*  Input Values         : None 
*  Output Values        : None
*  Return Value         : 0
*  Notes                :
*****************************************************************************/
void IFX_LLDP_GetRemoteMibPdu()
{
  uint8 ucaArr[6];
  uint32 uiaArr[6];
  uint8 uiaData[1500];
  int32 i;
  memset(uiaData, 0, 1500);
  for (i = 0; i < 6; i++)
  {
    fprintf(stdout, "\nEnter %dth element of MAC address (in HEX):", i);
    scanf("%x", &uiaArr[i]);
    ucaArr[i] = (uint8)uiaArr[i];
  }
  x_IFX_LLDP_MAC xMacAddr;
  memcpy(&(xMacAddr.uiaMacAddress), &(ucaArr[0]), 6);

  if(IFX_LLDP_PDU_REQUEST(xMacAddr, uiaData)==-1)
  {
    fprintf(stdout, "\nEntry Not Found\n");
  } else {
    fprintf(stdout, "\nRemote Data:\n");
    for (i = 0; i < 1500; i++)
      fprintf(stdout, "%x", uiaData[i]);
  }
}

/*****************************************************************************
*  Function Name        : IFX_LLDP_GetRemoteMibData
*  Description          : This function displays a remote LLDP TLVs in a
*                         structured format based upon the MAC address entered
*                         by the user.
*                           
*  Input Values         : None 
*  Output Values        : None
*  Return Value         : 0
*  Notes                :
*****************************************************************************/
void IFX_LLDP_GetRemoteMibData()
{
  x_IFX_LLDP_TLV_VALUES xTlvValues;
  uint32 uiaArr[6];
  x_IFX_LLDP_MAC xMacAddr;
  uint32 i;
  for (i = 0; i < 6; i++)
  {
    fprintf(stdout, "\nEnter %dth element of MAC address (in HEX):", i);
    scanf("%x", &uiaArr[i]);
    xMacAddr.uiaMacAddress[i] = (uint8)uiaArr[i];
  }
  memset(&xTlvValues, 0, sizeof(x_IFX_LLDP_TLV_VALUES));
  xTlvValues.uiNewTlv = IFX_LLDP_INTF_CHASSIS_ID;
  if(IFX_LLDP_MIB_REQUEST(xMacAddr, &xTlvValues)== -1)
  {
    fprintf(stdout, "\nEntry not found\n");
    return;
  } 
  uchar8 ucChassisIdTemp[256];
  memset(ucChassisIdTemp, '\0', sizeof(ucChassisIdTemp));
  memcpy(ucChassisIdTemp, xTlvValues.xValues.ucChassisId,
    sizeof(xTlvValues.xValues.ucChassisId));
  fprintf(stdout, "\nChassisId:%s\n", ucChassisIdTemp);

  memset(&xTlvValues, 0, sizeof(x_IFX_LLDP_TLV_VALUES));
  xTlvValues.uiNewTlv = IFX_LLDP_INTF_PORT_ID;
  IFX_LLDP_MIB_REQUEST(xMacAddr, &xTlvValues);

  uchar8 ucPortIdTemp[256];
  memset(ucPortIdTemp, '\0', sizeof(ucPortIdTemp));
  memcpy(ucPortIdTemp, xTlvValues.xValues.ucPortId,
    sizeof(xTlvValues.xValues.ucPortId));
  fprintf(stdout, "\nPortId: %s\n", ucPortIdTemp); 

  memset(&xTlvValues, 0, sizeof(x_IFX_LLDP_TLV_VALUES));
  xTlvValues.uiNewTlv = IFX_LLDP_INTF_MAC_PHY_CONFIGURATION_STATUS;
  IFX_LLDP_MIB_REQUEST(xMacAddr, &xTlvValues);

  fprintf(stdout, "MAC/PHY TLV:\n");
  fprintf(stdout, "Support: %d\n", xTlvValues.xValues.xMacPhyConfigurationStatus.uiSupport);
  fprintf(stdout, "Status:%d\n", xTlvValues.xValues.xMacPhyConfigurationStatus.uiStatus);
  fprintf(stdout, "Advertised Capabilities: %d\n", 
    xTlvValues.xValues.xMacPhyConfigurationStatus.uiAdvertisedCapabilities);
  fprintf(stdout, "MAU Type: %d\n", xTlvValues.xValues.xMacPhyConfigurationStatus.uiMauType);

  memset(&xTlvValues, 0, sizeof(x_IFX_LLDP_TLV_VALUES));
  xTlvValues.uiNewTlv = IFX_LLDP_INTF_SYSTEM_CAPABILITIES;
  IFX_LLDP_MIB_REQUEST(xMacAddr, &xTlvValues);

  fprintf(stdout, "\nSystem Capabilities TLV:\n");
  fprintf(stdout, "\nOther: %d", xTlvValues.xValues.xSystemCapabilities.bOther);
  fprintf(stdout, "\nRepeater: %d", xTlvValues.xValues.xSystemCapabilities.bRepeater);
  fprintf(stdout, "\nBridge: %d", xTlvValues.xValues.xSystemCapabilities.bBridge);
  fprintf(stdout, "\nWLANAp: %d", xTlvValues.xValues.xSystemCapabilities.bWlanAp);
  fprintf(stdout, "\nRouter: %d", xTlvValues.xValues.xSystemCapabilities.bRouter);
  fprintf(stdout, "\nTelephone: %d", xTlvValues.xValues.xSystemCapabilities.bTelephone);
  fprintf(stdout, "\nCableDevice: %d", xTlvValues.xValues.xSystemCapabilities.bCableDevice);
  fprintf(stdout, "\nStationOnly: %d", xTlvValues.xValues.xSystemCapabilities.bStationOnly);
  fprintf(stdout, "\nSystem Capability Supported:\n");
  fprintf(stdout, "\nOther Status: %d", xTlvValues.xValues.xSystemCapabilities.bOther_status);
  fprintf(stdout, "\nRepeater Status: %d", xTlvValues.xValues.xSystemCapabilities.bRepeater_status);
  fprintf(stdout, "\nBridge Status: %d", xTlvValues.xValues.xSystemCapabilities.bBridge_status);
  fprintf(stdout, "\nWLANAp Status: %d", xTlvValues.xValues.xSystemCapabilities.bWlanAp_status);
  fprintf(stdout, "\nRouter Status: %d", xTlvValues.xValues.xSystemCapabilities.bRouter_status);
  fprintf(stdout, "\nTelephone Status: %d", xTlvValues.xValues.xSystemCapabilities.bTelephone_status);
  fprintf(stdout, "\nCableDevice Status: %d", xTlvValues.xValues.xSystemCapabilities.bCableDevice_status);
  fprintf(stdout, "\nStationOnly Status: %d", xTlvValues.xValues.xSystemCapabilities.bStationOnly_status);

  memset(&xTlvValues, 0, sizeof(x_IFX_LLDP_TLV_VALUES));
  xTlvValues.uiNewTlv = IFX_LLDP_INTF_MED_CAPABILITIES;
  IFX_LLDP_MIB_REQUEST(xMacAddr, &xTlvValues);

  fprintf(stdout, "\nLLDP MED Capabilities TLV:\n");
  fprintf(stdout, "Capabilities: %d\n", xTlvValues.xValues.xLldpMedCapabilities.bCapabilities);
  fprintf(stdout, "Network Policy: %d\n", xTlvValues.xValues.xLldpMedCapabilities.bNetworkPolicy);
  fprintf(stdout, "ExtendedPowerMdiPD: %d\n", xTlvValues.xValues.xLldpMedCapabilities.bExtendedPowerMdiPD);
  fprintf(stdout, "Device Type: %d\n", xTlvValues.xValues.xLldpMedCapabilities.uiDeviceType);

  memset(&xTlvValues, 0, sizeof(x_IFX_LLDP_TLV_VALUES));
  xTlvValues.uiNewTlv = IFX_LLDP_INTF_NETWORK_POLICY;
  IFX_LLDP_MIB_REQUEST(xMacAddr, &xTlvValues);

  fprintf(stdout, "\nNetwork Policy TLV:\n");
  for (i = 0; i < xTlvValues.xValues.xNetworkPolicy.uiNoOfNetworkPolicyTlvs; i++)
  {
    fprintf(stdout, "Application Type: %d\n",
      xTlvValues.xValues.xNetworkPolicy.xNetworkPolicyArray[i].uiApplicationType);
    fprintf(stdout, "U: %d\n",
      xTlvValues.xValues.xNetworkPolicy.xNetworkPolicyArray[i].bUnknownFlag);
    fprintf(stdout, "T: %d\n",
      xTlvValues.xValues.xNetworkPolicy.xNetworkPolicyArray[i].bTaggedFlag);
    fprintf(stdout, "VLAN ID: %d\n",
      xTlvValues.xValues.xNetworkPolicy.xNetworkPolicyArray[i].uiVlanId);
    fprintf(stdout, "L2 Priority: %d\n",
      xTlvValues.xValues.xNetworkPolicy.xNetworkPolicyArray[i].uiL2Priority);
    fprintf(stdout, "DSCP Value %d\n\n",
      xTlvValues.xValues.xNetworkPolicy.xNetworkPolicyArray[i].uiDscpValue);
  }

  memset(&xTlvValues, 0, sizeof(x_IFX_LLDP_TLV_VALUES));
  xTlvValues.uiNewTlv = IFX_LLDP_INTF_EXTENDED_POWER_VIA_MDI;
  IFX_LLDP_MIB_REQUEST(xMacAddr, &xTlvValues);

  fprintf(stdout, "\nExtended Power Via MDI TLV:\n");
  fprintf(stdout, "Type: %d\n", xTlvValues.xValues.xExtendedPowerViaMdi.uiType);
  fprintf(stdout, "Source: %d\n", xTlvValues.xValues.xExtendedPowerViaMdi.uiSource);
  fprintf(stdout, "Priority: %d\n", xTlvValues.xValues.xExtendedPowerViaMdi.uiPriority);
  fprintf(stdout, "Value: %d\n", xTlvValues.xValues.xExtendedPowerViaMdi.uiValue);
}

/*****************************************************************************
*  Function Name        : IFX_LLDP_WaitForMsg
*  Description          :  This is a callback function which is called when a
*                          message is recevied from LLDP Agent.
*                           
*  Input Values         : None.
*  Output Values        : None.
*  Return Value         : None.
*  Notes                :
*****************************************************************************/
void IFX_LLDP_WaitForMsg(x_IFX_LLDP_LLDPAGENT_MSG *pxMsg)
{
      if (pxMsg->eMsgType == IFX_LLDP_PACKET_RXVD_MSG)
      {
        fprintf(stdout, "\nReceived Packet Information:\n");
        switch(pxMsg->ux_Data.xPacket.eDeviceType)
        {
          case IFX_LLDP_NCD:
            fprintf(stdout, "Destination Device Type: NCD\n");
            break;
          case IFX_LLDP_EPD:
            fprintf(stdout, "Destination Device Type: EPD\n");
            break;
          default:
            fprintf(stdout, "Destination Device Type: Unknown\n");
            break;
        }
        fprintf(stdout, "Destination MAC Address: %x:%x:%x:%x:%x:%x\n",
          pxMsg->ux_Data.xPacket.xMacAddress.uiaMacAddress[0],
          pxMsg->ux_Data.xPacket.xMacAddress.uiaMacAddress[1],
          pxMsg->ux_Data.xPacket.xMacAddress.uiaMacAddress[2],
          pxMsg->ux_Data.xPacket.xMacAddress.uiaMacAddress[3],
          pxMsg->ux_Data.xPacket.xMacAddress.uiaMacAddress[4],
          pxMsg->ux_Data.xPacket.xMacAddress.uiaMacAddress[5]);

        fprintf(stdout, "Packet Status: ");
        switch(pxMsg->ux_Data.xPacket.ePktStat)
        {
          case IFX_LLDP_PACKET_NEW:
            fprintf(stdout, "NEW\n");
            break;
          case IFX_LLDP_PACKET_EXISTING_CHANGED:
            fprintf(stdout, "Existing but changed\n");
            break;
          case IFX_LLDP_PACKET_EXISTING_UNCHANGED:
            fprintf(stdout, "Existing but unchanged\n");
            break;
          default:
            fprintf(stdout, "Invalid Packet status\n");
            break;
        }
        fprintf(stdout, "MIB Status: ");
        switch(pxMsg->ux_Data.xPacket.eMibStatus)
        {
          case IFX_LLDP_MIB_FREE:
            fprintf(stdout, "Memory Available\n");
            break;
          case IFX_LLDP_MIB_FULL:
            fprintf(stdout, "Memory Full\n");
            break;
          default:
            fprintf(stdout, "Invalid MIB status\n");
            break;
        }
      }
      else if (pxMsg->eMsgType == IFX_LLDP_PACKET_EXPIRED_MSG)
      {
        fprintf(stdout, "\nRemote Packet Expired...\n");
        switch(pxMsg->ux_Data.xPacketExpInfo.eDeviceType)
        {
          case IFX_LLDP_NCD:
            fprintf(stdout, "Destination Device Type: NCD\n");
            break;
          case IFX_LLDP_EPD:
            fprintf(stdout, "Destination Device Type: EPD\n");
            break;
          default:
            fprintf(stdout, "Destination Device Type: Unknown\n");
            break;
        }

        fprintf(stdout, "Destination MAC Address: %x:%x:%x:%x:%x:%x\n",
          pxMsg->ux_Data.xPacketExpInfo.xMacAddress.uiaMacAddress[0],
          pxMsg->ux_Data.xPacketExpInfo.xMacAddress.uiaMacAddress[1],
          pxMsg->ux_Data.xPacketExpInfo.xMacAddress.uiaMacAddress[2],
          pxMsg->ux_Data.xPacketExpInfo.xMacAddress.uiaMacAddress[3],
          pxMsg->ux_Data.xPacketExpInfo.xMacAddress.uiaMacAddress[4],
          pxMsg->ux_Data.xPacketExpInfo.xMacAddress.uiaMacAddress[5]);
      }
}

/*****************************************************************************
*  Function Name        : IFX_LLDP_GetData
*  Description          : This is a Wrapper function for scanf. return -1 if 
*                         'Enter' is pressed.
*                           
*  Input Values         : None.
*  Output Values        : Data entered by the user.
*  Return Value         : -1   - If 'Enter' is pressed.
*                         Data - If a valid input is provided.
*  Notes                :
*****************************************************************************/
int32 IFX_LLDP_GetData()
{
  int c, k = 0;
  char i[10];
  while((c = getchar()) != '\n')
  {
    i[k] = c;
    k++;
  }
  if (k == 0)
    return -1;
  else
  {
    i[k] = '\n';
    return atoi(i);
  }
}

/*****************************************************************************
*  Function Name        : IFX_LLDP_DisplayPortStats
*  Description          : This internal function fetches and displays port 
*                         statistics.
*                           
*  Input Values         : None.
*  Output Values        : NOne.
*  Return Value         : None.
*  Notes                :
*****************************************************************************/
void IFX_LLDP_DisplayPortStats()
{
  e_IFX_LLDP_PORT_NAME ePort;
  x_IFX_LLDP_PORT_STATS xStats;
  uint32 uiPort;
  fprintf(stdout, "\nEnter Port Id (0 - PC, 1 - LAN): ");
  scanf("%d", &uiPort);
  if ((uiPort != IFX_LLDP_PC) && (uiPort != IFX_LLDP_LAN))
  {
    fprintf(stdout, "\nInvalid Port");
    return;
  }
  ePort = uiPort;
  if (IFX_LLDP_COUNTERS_REQUEST (ePort, &xStats) == -1)
  {
    fprintf(stdout, "\nFetching Stats failed\n");
    return;
  }
  fprintf(stdout, "\nStatistics:\n");
  fprintf(stdout, "Agedout Packets      : %d\n", xStats.ulPortStatsAgeoutsTotal);
  fprintf(stdout, "Packets Discardrd    : %d\n", xStats.ulPortStatsFramesDiscardedTotal);
  fprintf(stdout, "Erroneous Packets    : %d\n", xStats.ulPortStatsFramesInErrorsTotal);
  fprintf(stdout, "LLDP Packets Received: %d\n", xStats.ulPortStatsFramesInTotal);
  fprintf(stdout, "LLDP Packets Sent    : %d\n", xStats.ulPortStatsFramesOutTotal);
  fprintf(stdout, "TLVs Discardrd       : %d\n", xStats.ulPortStatsTLVsDiscardedTotal);
  fprintf(stdout, "Unrecognised TLVs    : %d\n", 
    xStats.ulPortStatsTLVsUnrecognizedTotal);
}
